/**
 * EasyBeans
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: OSGiBean.java 4340 2008-12-03 21:44:57Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.examples.osgi;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Properties;

import javax.ejb.Stateless;
import javax.interceptor.AroundInvoke;
import javax.interceptor.InvocationContext;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Constants;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceReference;
import org.osgi.service.cm.Configuration;
import org.osgi.service.cm.ConfigurationAdmin;
import org.osgi.service.log.LogService;
import org.ow2.easybeans.osgi.annotation.BindingMode;
import org.ow2.easybeans.osgi.annotation.OSGiResource;

/**
 * EJB that can access the OSGi world.
 * @author Florent BENOIT
 */
@Stateless(mappedName="OSGiBean")
public class OSGiBean implements SimpleInterface {


    /**
     * Bundle context that will be injected by the container.
     */
    @OSGiResource
    private final BundleContext bundleContextField = null;

    /**
     * Bundle context that will be injected by the container by a setter method.
     */
    private BundleContext bundleContextMethod = null;

    /**
     * This dependency will be managed (injected/outjected) by OSGi in a method.
     */
    private ISimpleDependency dependencyMethod;

    /**
     * This OSGi dependency is directly injected/outjected in the field.
     * Notice that this interface is implemented by an EJB.
     */
    @OSGiResource
    private ISimpleDependency dependencyField;

    /**
     * This OSGi dependency is directly injected/outjected in the field.
     * Notice that this interface is implemented by a legacy OSGi service.
     */
    @OSGiResource
    private LogService logger;

    @OSGiResource
    private ConfigurationAdmin configAdmin;

    /**
     * Sets the bundleContext provided by the container.
     * @param bundleContext the given bundle context
     */
    @OSGiResource
    public void setBundleContext(final BundleContext bundleContext) {
        System.out.println("Receiving bundlecontext from the container:" + bundleContext);
        this.bundleContextMethod = bundleContext;
    }

    /**
     * @return the list of all OSGi Bundles symbolic names found on the gateway.
     */
    public List<String> listOSGiSymbolicNameBundles() {
        List<String> symbolicNames = new ArrayList<String>();
        Bundle[] bundles = bundleContextMethod.getBundles();
        if (bundles != null) {
            for (Bundle bundle : bundles) {
                symbolicNames.add(bundle.getSymbolicName());
            }
        }
        return symbolicNames;
    }

    /**
     * Try to log on the OSGi Log service.
     * @return result of the commands.
     */
    public String tryLogService() {
        StringBuilder sb = new StringBuilder();
        if (logger == null) {
            sb.append("No log service reference found");
        } else {
            sb.append("Found log service reference '" + logger + "'.");
            logger.log(LogService.LOG_INFO, "This is a test of a log message from an EJB 3.0 on an OSGi service");
            sb.append("Log on the logservice has been done");
        }
        // Display log on the server side
        System.out.println(sb.toString());
        return sb.toString();
    }

    /**
     * See how many services of the given class have been registered in OSGi.
     *
     * @param objectClass service interface name
     * @return the number of services (matching the objectClass parameter) registered in OSGi
     */
    public int getNumberOfRegisteredServices(final String objectClass) {

        ServiceReference[] references = null;
        try {
            references = bundleContextField.getServiceReferences(objectClass, null);
        } catch (InvalidSyntaxException e) {
            // Should not happen
            logger.log(LogService.LOG_ERROR, "Wrong LDAP filter syntax");
        }
        return references.length;

    }

    /**
     * EJB 3.0 interceptor.
     * @param invocationContext the invocation context of the EJB.
     * @return the result of the invocation.
     * @throws Exception if the interceptor is failing
     */
    @AroundInvoke
    protected Object intercepted(final InvocationContext invocationContext) throws Exception {
        System.out.println("EJB 3.0 interceptor on the method '" + invocationContext.getMethod() + "'.");
        return invocationContext.proceed();
    }

    /**
     * Injection method for the {@link ISimpleDependency} interface.
     * @param dep the bean implementing the interface
     */
    @OSGiResource(mode=BindingMode.BIND)
    public void bindDependency(final ISimpleDependency dep) {
        System.out.println(ISimpleDependency.class.getName() + " has been injected.");
        this.dependencyMethod = dep;
    }

    /**
     * Outjection method for the {@link ISimpleDependency} interface.
     * @param dep the bean implementing the interface
     */
    @OSGiResource(mode=BindingMode.UNBIND)
    public void unbindLogService(final ISimpleDependency dep) {
        System.out.println(ISimpleDependency.class.getName() + " has been removed.");
        this.dependencyMethod = null;
    }

    /**
     * A test method checking if the dependency has been injected in the field.
     * @return <code>true</code> if injected
     */
    public boolean isDependencyFieldInjected() {
        return dependencyField != null;
    }

    /**
     * A test method checking if the dependency has been injected through the methods.
     * @return <code>true</code> if injected
     */
    public boolean isDependencyMethodInjected() {
        return dependencyMethod != null;
    }

    /**
     * This method throws a runtime exception.
     */
    public void throwIllegalStateException() {
        throw new IllegalStateException();
    }

    /**
     * @param pid Service PID
     * @param propertyName property name to be retrieve
     * @return the value of the property associated with te given pid (may return null).
     */
    public String getServiceProperty(final String pid, final String propertyName) {
        String filter = "(" + Constants.SERVICE_PID + "=" + pid + ")";
        ServiceReference[] references = null;
        try {
            references = bundleContextField.getServiceReferences(null, filter);
        } catch (InvalidSyntaxException e) {
            // Should never happen
            return null;
        }
        if (references == null) {
            return null;
        }
        ServiceReference ref = references[0];

        String value = (String) ref.getProperty(propertyName);
        return value;
    }

    /**
     * Update the service with the given properties set.
     * @param properties new service properties
     */
    @SuppressWarnings("unchecked")
    public void updateConfigurationProperties(final String pid, final Properties properties) {
        Configuration configuration;
        try {
            configuration = configAdmin.getConfiguration(pid, null);
        } catch (IOException e) {
            throw new IllegalStateException("Cannot create the Configuration " + pid, e);
        }

        Dictionary dict = new Hashtable();
        for(Enumeration en = properties.propertyNames(); en.hasMoreElements(); ) {
            String name = (String) en.nextElement();
            dict.put(name, properties.getProperty(name));
        }

        try {
            configuration.update(dict);
        } catch (IOException e) {
            throw new IllegalStateException("Cannot update the Configuration " + pid, e);
        }
    }

    /**
     * @param containerSymbolicName symbolic name of the EjbJar bundle
     * @return <code>true</code> if the EZBContainer service with the
     *         given name has been found.
     */
	public boolean isEZBContainerAvailable(String containerSymbolicName) {
        ServiceReference[] references = null;
        try {
        	String filter = "(name=" + containerSymbolicName + ")";
            references = bundleContextField.getServiceReferences("org.ow2.easybeans.api.EZBContainer", filter);
        } catch (InvalidSyntaxException e) {
            // Should not happen
            logger.log(LogService.LOG_ERROR, "Wrong LDAP filter syntax");
        }
        return references != null;
	}
}
